/* $Workfile: OsaCon.h $   $Revision: 18 $    $Modtime: 11/04/04 10:09a $  */
#ifndef OSA_CON_H
#define OSA_CON_H
/*******************************************************************************

  @doc PublicHwApi OsaCon

  Copyright (C) 1999-2003 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module
  OsaCon | This file contains prototypes for platform the generic 
  implementation of some console functions.

  @contents2
  OsaCon.h |

  @index func | OsaCon

  @contents2
  Generic Console Functions - OsaConsoleXXX() | A platform generic 
  implementation of some console functions.
  
  @head3 OsaConsoleXXX() Function Reference |

  @index func | OsaCon

*******************************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif

#if defined(OSA_DRIVER)
#error "This file should not be included in kernel mode code"
#endif

#if defined(__linux__)

#include <stdarg.h>
#if 1
#include <ncurses.h>
#else
#include <stdio.h>
#define getch getchar
#endif

#elif defined(_WIN32_WCE)
#include <windows.h>

#elif defined(WIN32)
#include <windows.h>
#include <conio.h>

#else
#error "Platform not supported"

#endif


/******************************************************************************

  @func
    Initialize the platform generic Osa Console functions.

  @comm
    This function must be called before any other Osa function can be called. 
    Osa functions are used by the demo programs.

  @ex |

    // first OTX function call for application if other Osa functions are used 
    OsaConsoleInit();
    ...

    // this is called before application exits
    OsaConsoleExit();

  @xref
    <f OsaConsoleExit>

******************************************************************************/
OTX_EXPORT void OTX_API OsaConsoleInit(void);


/******************************************************************************

  @func
    Exit the platform generic functions.

  @comm
    This function should be called before exiting the application if 
    <f OsaConsoleInit> was called.

    Osa functions are used by the demo programs.
   
    see example on <f OsaConsoleInit>

  @xref
    <f OsaConsoleInit>

******************************************************************************/
OTX_EXPORT void OTX_API OsaConsoleExit(void);


/******************************************************************************

  @func
    Platform generic Osa implementation of the standard printf() function.

  @comm
    Osa functions are used by the demo programs.

  @ex |

    // Usually it is used in function DemoLogf() in demo programs 
    static void DemoLogf(char *formatStr, ...)
    {
      va_list argList;
      va_start(argList, formatStr);

      OsaConsolePrintF(formatStr, argList);
      if (g_pDemoLogFile) 
        vfprintf(g_pDemoLogFile,   formatStr, argList);

      va_end(argList);
    }


******************************************************************************/
OTX_EXPORT int OTX_API OsaConsolePrintF(
  char *formatStr,          // @parm Format string. See printf() in stdio.h
  va_list argList           // @parm Argument list. See printf() in stdio.h
);


/******************************************************************************

  @func
    Platform generic Osa implementation of the standard gets() function.

  @comm
    Osa functions are used by the demo programs.

  @ex |
      char strBuf[12];
      static OTX_UINT32 nAddress = 0;

      // get input "nAddress" from key board
      OsaConsoleGetStr(strBuf); 
      if (*strBuf) {
        nAddress = strtoul(strBuf, NULL, 0);
      }


******************************************************************************/
OTX_EXPORT char * OTX_API OsaConsoleGetStr(
  char *buffer              // @parm Placeholder for input string. See gets() in stdio.h
);


/******************************************************************************

  @func
    Platform generic Osa implementation of a standard beep() function.

  @comm
    Osa functions are used by the demo programs.

  @ex |
    // beep 200 ms in 4000 Hz
    OsaBeep(4000, 200);

******************************************************************************/
OTX_EXPORT void OTX_API OsaBeep(
  unsigned int Freq,        // @parm sound frequency, in hertz
  unsigned int msDuration   // @parm sound duration, in milliseconds
);


/******************************************************************************

  @func
    Platform generic Osa implementation of the standard puts() function.

  @comm
    Osa functions are used by the demo programs.

  @xref
    <f OsaConsoleInit>

  @ex |
    // write "Hello Odin!" on the screen
    OsaConsolePutStr( "Hello Odin!");



******************************************************************************/
OTX_EXPORT int OTX_API OsaConsolePutStr(
  char *buffer              // @parm Placeholder for input string. See puts() in stdio.h
);


/******************************************************************************

  @func
    Platform generic Osa implementation of the standard getch() function.

  @comm
    Osa functions are used by the demo programs.

  @ex |
  
    OTX_INT32 c;
    
    // get a character from key board
    c = OsaConsoleGetCh();

******************************************************************************/
OTX_EXPORT int OTX_API OsaConsoleGetCh(
  void
);


/******************************************************************************

  @func
    Provides an alternative input stream to the console functions other
    than from the keyboard buffer. The string passed to this function will be 
    fed to <f OsaConsoleGetStr> and <f OsaConsoleGetCh>. 
    
    To indicate an 'Enter' key in the <f OsaConsoleGetStr> function, use 
    the '^' character in the parameter to this function.

  @comm
    Osa functions are used by the demo programs.

  @ex | 
    
    // it is usually used at the begining of application to get input stream
    // from command line
    if (g_demoEnv.pszInputStr) {
      OsaConsoleInput(g_demoEnv.pszInputStr);
    }

  @xref
    <f OsaConsoleGetStr>
    <f OsaConsoleGetCh>

******************************************************************************/
OTX_EXPORT void OTX_API OsaConsoleInput(
  char *inputStr
);


#if defined(_WIN32_WCE)

#if defined(_TURNSTONE_)
// replacement library routines not in wince 2x
__declspec(dllexport) long strtol(const char *nptr, char **endptr, int base);
__declspec(dllexport) unsigned long strtoul(const char *nptr, char **endptr, int base);
__declspec(dllexport) char *strrchr(const char *string, int c);
#endif  // #if defined(_TURNSTONE_)

/******************************************************************************

  @func
    Provides an mechanism to have the equivalent of a console application
    under Windows CE.  This routine is called from WinMain, and processes
    the command line parameters, if any, starts up a thread routine (passed
    as a function pointer to what would be a main(int argc, char *argv[]) in
    a normal console application, and provides the requirement of Windows
    message handling:

    while (GetMessage(&msg, NULL, 0, 0) == TRUE) {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }


  @comm
    Osa functions are used by the demo programs.

  @ex | 
    
    int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nShowCmd) 
    {
      // default command line parameters
      static char exename[OTX_MAX_PATH] = "";
      static char *cmdParams[] = { exename, "/TM", NULL }; 
      char *pc;

      GetModuleFileName(NULL, (LPTSTR) cmdParams[0], OTX_MAX_PATH);
      strcpy(g_demoEnv.cwd, cmdParams[0]);
      pc = strrchr(g_demoEnv.cwd, '\\');
      if (pc != NULL) {
        *++pc = '\0';
      }

      return OsaWinCeConsoleMain(hInstance, hPrevInstance, lpCmdLine, nShowCmd,
                  cmdParams, sizeof(cmdParams) / sizeof(cmdParams[0]), main);
    }


******************************************************************************/
OTX_EXPORT int OTX_API OsaWinCeConsoleMain(
  HINSTANCE hInstance,
  HINSTANCE hPrevInstance,
  LPWSTR lpCmdLine,
  int nShowCmd,
  char **defargv,
  int defargc,
  int (*pmain)(int argc, char **argv)
);

#endif  // #if defined(_WIN32_WCE)

#ifdef __cplusplus
  } // extern "C"
#endif

/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1999-2003

  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/

#endif // OSA_CON_H

